searchData={"content_type":"text/markdown","items":[{"doc":"","ref":"Rail.html","title":"Rail","type":"module"},{"doc":"Apply a function or pipe to a function call when value is not {:error, _} or :error","ref":"Rail.html#%3E%3E%3E/2","title":"Rail.>>>/2","type":"macro"},{"doc":"iex> 1 >>> fn v -> Integer.to_string(v) end\n    \"1\"\n    iex> {:ok, 1} >>> fn v -> Integer.to_string(v) end\n    \"1\"\n    iex> {:ok, 1} >>> Integer.to_string()\n    \"1\"\n    iex> :error >>> Integer.to_string()\n    :error\n    iex> {:error, :div_by_zero} >>> Integer.to_string()\n    {:error, :div_by_zero}","ref":"Rail.html#%3E%3E%3E/2-examples","title":"Examples - Rail.>>>/2","type":"macro"},{"doc":"Apply a function when value is not {:error, _} or :error","ref":"Rail.html#chain/2","title":"Rail.chain/2","type":"function"},{"doc":"iex> 1 |> Rail.chain(fn v -> v + 10 end)\n    11\n    iex> {:ok, 1} |> Rail.chain(fn v -> v + 10 end)\n    11\n    iex> :error |> Rail.chain(fn v -> v + 10 end)\n    :error\n    iex> {:error, :noent} |> Rail.chain(fn v -> v + 10 end)\n    {:error, :noent}","ref":"Rail.html#chain/2-examples","title":"Examples - Rail.chain/2","type":"function"},{"doc":"Similar to `Kernel.def/2`, but the function body is wrapped in a `rail/1` block.\n\n```elixir\ndefmodule Target do\n  use Rail\n\n  def sum() do\n    x <- {:ok, 1}\n    y <- {:ok, 1}\n\n    {:ok, x + y}\n  end\nend\n```","ref":"Rail.html#def/2","title":"Rail.def/2","type":"macro"},{"doc":"Similar to `Kernel.defp/2`, but the function body is wrapped in a `rail/1` block.\n\n```elixir\ndefmodule Target do\n  use Rail\n\n  defp sum() do\n    x <- {:ok, 1}\n    y <- {:ok, 1}\n\n    {:ok, x + y}\n  end\nend\n```","ref":"Rail.html#defp/2","title":"Rail.defp/2","type":"macro"},{"doc":"Return a new tuple for error of {:error, error}, otherwise bypass","ref":"Rail.html#flat_map_error/2","title":"Rail.flat_map_error/2","type":"function"},{"doc":"iex> :error |> Rail.flat_map_error(fn :noent -> {:noreply, :not_found} end)\n    :error\n    iex> {:error, :noent} |> Rail.flat_map_error(fn :noent -> {:noreply, :not_found} end)\n    {:noreply, :not_found}\n    iex> {:ok, 1} |> Rail.flat_map_error(fn :noent -> {:noreply, :not_found} end)\n    {:ok, 1}","ref":"Rail.html#flat_map_error/2-examples","title":"Examples - Rail.flat_map_error/2","type":"function"},{"doc":"Return a new tuple for value of {:ok, value}, otherwise bypass","ref":"Rail.html#flat_map_ok/2","title":"Rail.flat_map_ok/2","type":"function"},{"doc":"iex> :ok |> Rail.flat_map_ok(fn 1 -> {:noreply, 10} end)\n    :ok\n    iex> {:ok, 1} |> Rail.flat_map_ok(fn 1 -> {:noreply, 10} end)\n    {:noreply, 10}\n    iex> {:error, 1} |> Rail.flat_map_ok(fn 1 -> {:noreply, 10} end)\n    {:error, 1}","ref":"Rail.html#flat_map_ok/2-examples","title":"Examples - Rail.flat_map_ok/2","type":"function"},{"doc":"Apply a function for error of {:error, error}, otherwise bypass","ref":"Rail.html#map_error/2","title":"Rail.map_error/2","type":"function"},{"doc":"iex> :error |> Rail.map_error(fn :noent -> :not_found end)\n    :error\n    iex> {:error, :noent} |> Rail.map_error(fn :noent -> :not_found end)\n    {:error, :not_found}\n    iex> {:ok, 1} |> Rail.map_error(fn :noent -> :not_found end)\n    {:ok, 1}","ref":"Rail.html#map_error/2-examples","title":"Examples - Rail.map_error/2","type":"function"},{"doc":"Apply a function for value of {:ok, value}, otherwise bypass","ref":"Rail.html#map_ok/2","title":"Rail.map_ok/2","type":"function"},{"doc":"iex> :ok |> Rail.map_ok(fn 1 -> 10 end)\n    :ok\n    iex> {:ok, 1} |> Rail.map_ok(fn 1 -> 10 end)\n    {:ok, 10}\n    iex> {:error, 1} |> Rail.map_ok(fn 1 -> 10 end)\n    {:error, 1}","ref":"Rail.html#map_ok/2-examples","title":"Examples - Rail.map_ok/2","type":"function"},{"doc":"Normalize input to {:ok, value} or {:error, error}","ref":"Rail.html#normalize/1","title":"Rail.normalize/1","type":"function"},{"doc":"iex> :error |> Rail.normalize()\n    {:error, nil}\n    iex> {:error, 1, 2} |> Rail.normalize()\n    {:error, {1, 2}}\n    iex> :ok |> Rail.normalize()\n    {:ok, nil}\n    iex> {:ok, 1, 2} |> Rail.normalize()\n    {:ok, {1, 2}}\n    iex> {:hello, :world} |> Rail.normalize()\n    {:ok, {:hello, :world}}","ref":"Rail.html#normalize/1-examples","title":"Examples - Rail.normalize/1","type":"function"},{"doc":"Introduces new syntax `left <- right`,\n\n- which bind `value` to left when `right` is `{:ok, value}` or just `value`\n- or skips entire code block when `right` is `{:error, _}` or `:error`.","ref":"Rail.html#rail/1","title":"Rail.rail/1","type":"macro"},{"doc":"iex> rail do\n    ...>   x     y  \n    ...>   x + y\n    ...> end\n    3","ref":"Rail.html#rail/1-examples","title":"Examples - Rail.rail/1","type":"macro"},{"doc":"Similar to `Kernel.def/2`, but the function body is wrapped in a `rail/1` block.\n\n```elixir\ndefmodule Target do\n  use Rail\n\n  rail sum() do\n    x <- {:ok, 1}\n    y <- {:ok, 1}\n\n    {:ok, x + y}\n  end\nend\n```","ref":"Rail.html#rail/2","title":"Rail.rail/2","type":"macro"},{"doc":"Similar to `Kernel.defp/2`, but the function body is wrapped in a `rail/1` block.\n\n```elixir\ndefmodule Target do\n  use Rail\n\n  rail sum() do\n    x <- {:ok, 1}\n    y <- {:ok, 1}\n\n    {:ok, x + y}\n  end\nend\n```","ref":"Rail.html#railp/2","title":"Rail.railp/2","type":"macro"},{"doc":"","ref":"Rail.ErrorReporter.html","title":"Rail.ErrorReporter","type":"behaviour"},{"doc":"","ref":"Rail.ErrorReporter.html#c:report_error/2","title":"Rail.ErrorReporter.report_error/2","type":"callback"},{"doc":"# Rail\n\n[![test](https://github.com/jechol/rail/actions/workflows/test.yml/badge.svg)](https://github.com/jechol/rail/actions/workflows/test.yml)\n[![Coverage Status](https://coveralls.io/repos/github/jechol/rail/badge.svg?branch=main)](https://coveralls.io/github/jechol/rail?branch=main)\n[![Hex.pm](https://img.shields.io/hexpm/v/rail)](https://hex.pm/packages/rail)\n[![GitHub](https://img.shields.io/github/license/jechol/rail)](https://github.com/jechol/rail/blob/main/LICENSE)\n\n`Rail` is a helper macros for \"Railway oriented programming\".\n\nIt helps you handle error cases at almost no cost with `rail`, `>>>`, and `def` macro.\n\nIf you are not comfortable with \"Railway oriented programming\", see [Railway oriented programming](https://www.youtube.com/watch?v=fYo3LN9Vf_M)","ref":"readme.html","title":"Rail","type":"extras"},{"doc":"```elixir\ndef deps do\n  [\n    {:rail, \"~> 1.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Rail","type":"extras"},{"doc":"#","ref":"readme.html#usage","title":"Usage - Rail","type":"extras"},{"doc":"`use Rail` introduces new syntax `left   Calc.div(10, 2)\n5.0\niex> Calc.div(10, 0)\n{:error, :div_by_zero}\n```\n\n`rail/1` is available inside other code blocks.\n\n```elixir\n\niex> rail do\n...    x  >> right` is similar to `|>`, but\n\n- applies right function only when left is `value` or `{:ok, value}`.\n- support both function and call expression, which makes it compatible with `|>`.\n\n```elixir\niex> 1 >>> fn v -> Integer.to_string(v) end\n\"1\"\niex> {:ok, 1} >>> fn v -> Integer.to_string(v) end\n\"1\"\niex> {:ok, 1} >>> Integer.to_string()\n\"1\"\niex> {:error, :div_by_zero} >>> Integer.to_string()\n{:error, :div_by_zero}\n```\n\nFor more examples, see `Rail`","ref":"readme.html#basic-usage","title":"Basic usage - Rail","type":"extras"},{"doc":"[MIT](./LICENSE)","ref":"readme.html#license","title":"LICENSE - Rail","type":"extras"},{"doc":"MIT License\n\nCopyright (c) 2022 Jechol Lee\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]}